@@ -15,6 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use crate::arith::derive_arith;
 use crate::bigint::div::div_rem;
 use num::cast::AsPrimitive;
 use num::{BigInt, FromPrimitive, ToPrimitive};
@@ -53,6 +54,7 @@ enum DivRemError {
 /// A signed 256-bit integer
 #[allow(non_camel_case_types)]
 #[derive(Copy, Clone, Default, Eq, PartialEq, Hash)]
+#[repr(C)]
 pub struct i256 {
     low: u128,
     high: i128,
@@ -214,6 +216,7 @@ impl i256 {
         }
     }
 
+    /// Create an `i256` value from a 128-bit value.
     pub const fn from_i128(v: i128) -> Self {
         Self::from_parts(v as u128, v >> 127)
     }
@@ -638,55 +641,53 @@ fn mulx(a: u128, b: u128) -> (u128, u128) {
     (low, high)
 }
 
-macro_rules! derive_op {
-    ($t:ident, $op:ident, $wrapping:ident, $checked:ident) => {
-        impl std::ops::$t for i256 {
-            type Output = i256;
+derive_arith!(
+    i256,
+    Add,
+    AddAssign,
+    add,
+    add_assign,
+    wrapping_add,
+    checked_add
+);
+derive_arith!(
+    i256,
+    Sub,
+    SubAssign,
+    sub,
+    sub_assign,
+    wrapping_sub,
+    checked_sub
+);
+derive_arith!(
+    i256,
+    Mul,
+    MulAssign,
+    mul,
+    mul_assign,
+    wrapping_mul,
+    checked_mul
+);
+derive_arith!(
+    i256,
+    Div,
+    DivAssign,
+    div,
+    div_assign,
+    wrapping_div,
+    checked_div
+);
+derive_arith!(
+    i256,
+    Rem,
+    RemAssign,
+    rem,
+    rem_assign,
+    wrapping_rem,
+    checked_rem
+);
 
-            #[cfg(debug_assertions)]
-            fn $op(self, rhs: Self) -> Self::Output {
-                self.$checked(rhs).expect("i256 overflow")
-            }
-
-            #[cfg(not(debug_assertions))]
-            fn $op(self, rhs: Self) -> Self::Output {
-                self.$wrapping(rhs)
-            }
-        }
-
-        impl<'a> std::ops::$t<i256> for &'a i256 {
-            type Output = i256;
-
-            fn $op(self, rhs: i256) -> Self::Output {
-                (*self).$op(rhs)
-            }
-        }
-
-        impl<'a> std::ops::$t<&'a i256> for i256 {
-            type Output = i256;
-
-            fn $op(self, rhs: &'a i256) -> Self::Output {
-                self.$op(*rhs)
-            }
-        }
-
-        impl<'a, 'b> std::ops::$t<&'b i256> for &'a i256 {
-            type Output = i256;
-
-            fn $op(self, rhs: &'b i256) -> Self::Output {
-                (*self).$op(*rhs)
-            }
-        }
-    };
-}
-
-derive_op!(Add, add, wrapping_add, checked_add);
-derive_op!(Sub, sub, wrapping_sub, checked_sub);
-derive_op!(Mul, mul, wrapping_mul, checked_mul);
-derive_op!(Div, div, wrapping_div, checked_div);
-derive_op!(Rem, rem, wrapping_rem, checked_rem);
-
-impl std::ops::Neg for i256 {
+impl Neg for i256 {
     type Output = i256;
 
     #[cfg(debug_assertions)]
@@ -838,9 +839,8 @@ impl ToPrimitive for i256 {
 #[cfg(all(test, not(miri)))] // llvm.x86.subborrow.64 not supported by MIRI
 mod tests {
     use super::*;
-    use num::{BigInt, FromPrimitive, Signed, ToPrimitive};
+    use num::Signed;
     use rand::{thread_rng, Rng};
-    use std::ops::Neg;
 
     #[test]
     fn test_signed_cmp() {
