@@ -17,6 +17,10 @@
 
 //! Low-level buffer abstractions for [Apache Arrow Rust](https://docs.rs/arrow)
 
+// used by [`buffer::mutable::dangling_ptr`]
+#![cfg_attr(miri, feature(strict_provenance))]
+#![warn(missing_docs)]
+
 pub mod alloc;
 pub mod buffer;
 pub use buffer::*;
@@ -25,10 +29,17 @@ pub mod builder;
 pub use builder::*;
 
 mod bigint;
-mod bytes;
-mod native;
 pub use bigint::i256;
 
+mod bytes;
+
+mod native;
 pub use native::*;
+
 mod util;
 pub use util::*;
+
+mod interval;
+pub use interval::*;
+
+mod arith;
