@@ -271,6 +271,21 @@ impl FFI_ArrowArray {
         self.null_count as usize
     }
 
+    /// Returns the null count, checking for validity
+    #[inline]
+    pub fn null_count_opt(&self) -> Option<usize> {
+        usize::try_from(self.null_count).ok()
+    }
+
+    /// Set the null count of the array
+    ///
+    /// # Safety
+    /// Null count must match that of null buffer
+    #[inline]
+    pub unsafe fn set_null_count(&mut self, null_count: i64) {
+        self.null_count = null_count;
+    }
+
     /// Returns the buffer at the provided index
     ///
     /// # Panic
