@@ -52,8 +52,8 @@ where
     Tz: TimeZone + Send,
 {
     fn decode(&mut self, tape: &Tape<'_>, pos: &[u32]) -> Result<ArrayData, ArrowError> {
-        let mut builder = PrimitiveBuilder::<P>::with_capacity(pos.len())
-            .with_data_type(self.data_type.clone());
+        let mut builder =
+            PrimitiveBuilder::<P>::with_capacity(pos.len()).with_data_type(self.data_type.clone());
 
         for p in pos {
             match tape.get(*p) {
@@ -71,14 +71,12 @@ where
                         TimeUnit::Second => date.timestamp(),
                         TimeUnit::Millisecond => date.timestamp_millis(),
                         TimeUnit::Microsecond => date.timestamp_micros(),
-                        TimeUnit::Nanosecond => {
-                            date.timestamp_nanos_opt().ok_or_else(|| {
-                                ArrowError::ParseError(format!(
-                                    "{} would overflow 64-bit signed nanoseconds",
-                                    date.to_rfc3339(),
-                                ))
-                            })?
-                        }
+                        TimeUnit::Nanosecond => date.timestamp_nanos_opt().ok_or_else(|| {
+                            ArrowError::ParseError(format!(
+                                "{} would overflow 64-bit signed nanoseconds",
+                                date.to_rfc3339(),
+                            ))
+                        })?,
                     };
                     builder.append_value(value)
                 }
@@ -99,7 +97,7 @@ where
                 TapeElement::I32(v) => builder.append_value(v as i64),
                 TapeElement::I64(high) => match tape.get(p + 1) {
                     TapeElement::I32(low) => {
-                        builder.append_value((high as i64) << 32 | low as i64)
+                        builder.append_value((high as i64) << 32 | (low as u32) as i64)
                     }
                     _ => unreachable!(),
                 },
