@@ -197,6 +197,17 @@ pub enum DataType {
     /// DataType::Timestamp(TimeUnit::Second, Some("string".to_string().into()));
     /// ```
     ///
+    /// # Timezone representation
+    /// ----------------------------
+    /// It is possible to use either the timezone string representation, such as "UTC", or the absolute time zone offset "+00:00".
+    /// For timezones with fixed offsets, such as "UTC" or "JST", the offset representation is recommended, as it is more explicit and less ambiguous.
+    ///
+    /// Most arrow-rs functionalities use the absolute offset representation,
+    /// such as [`PrimitiveArray::with_timezone_utc`] that applies a
+    /// UTC timezone to timestamp arrays.
+    ///
+    /// [`PrimitiveArray::with_timezone_utc`]: https://docs.rs/arrow/latest/arrow/array/struct.PrimitiveArray.html#method.with_timezone_utc
+    ///
     /// Timezone string parsing
     /// -----------------------
     /// When feature `chrono-tz` is not enabled, allowed timezone strings are fixed offsets of the form "+09:00", "-09" or "+0930".
@@ -234,7 +245,7 @@ pub enum DataType {
     ///
     /// # Recommendation
     ///
-    /// Users should prefer [`DataType::Date32`] to cleanly represent the number
+    /// Users should prefer [`Date32`] to cleanly represent the number
     /// of days, or one of the Timestamp variants to include time as part of the
     /// representation, depending on their use case.
     ///
@@ -242,6 +253,7 @@ pub enum DataType {
     ///
     /// For more details, see [#5288](https://github.com/apache/arrow-rs/issues/5288).
     ///
+    /// [`Date32`]: Self::Date32
     /// [Schema.fbs]: https://github.com/apache/arrow/blob/main/format/Schema.fbs
     Date64,
     /// A signed 32-bit time representing the elapsed time since midnight in the unit of `TimeUnit`.
@@ -271,10 +283,12 @@ pub enum DataType {
     LargeBinary,
     /// Opaque binary data of variable length.
     ///
-    /// Logically the same as [`Self::Binary`], but the internal representation uses a view
+    /// Logically the same as [`Binary`], but the internal representation uses a view
     /// struct that contains the string length and either the string's entire data
     /// inline (for small strings) or an inlined prefix, an index of another buffer,
     /// and an offset pointing to a slice in that buffer (for non-small strings).
+    ///
+    /// [`Binary`]: Self::Binary
     BinaryView,
     /// A variable-length string in Unicode with UTF-8 encoding.
     ///
@@ -288,10 +302,12 @@ pub enum DataType {
     LargeUtf8,
     /// A variable-length string in Unicode with UTF-8 encoding
     ///
-    /// Logically the same as [`Self::Utf8`], but the internal representation uses a view
+    /// Logically the same as [`Utf8`], but the internal representation uses a view
     /// struct that contains the string length and either the string's entire data
     /// inline (for small strings) or an inlined prefix, an index of another buffer,
     /// and an offset pointing to a slice in that buffer (for non-small strings).
+    ///
+    /// [`Utf8`]: Self::Utf8
     Utf8View,
     /// A list of some logical data type with variable length.
     ///
@@ -300,11 +316,12 @@ pub enum DataType {
 
     /// (NOT YET FULLY SUPPORTED)  A list of some logical data type with variable length.
     ///
+    /// Logically the same as [`List`], but the internal representation differs in how child
+    /// data is referenced, allowing flexibility in how data is layed out.
+    ///
     /// Note this data type is not yet fully supported. Using it with arrow APIs may result in `panic`s.
     ///
-    /// The ListView layout is defined by three buffers:
-    /// a validity bitmap, an offsets buffer, and an additional sizes buffer.
-    /// Sizes and offsets are both 32 bits for this type
+    /// [`List`]: Self::List
     ListView(FieldRef),
     /// A list of some logical data type with fixed length.
     FixedSizeList(FieldRef, i32),
@@ -315,11 +332,12 @@ pub enum DataType {
 
     /// (NOT YET FULLY SUPPORTED)  A list of some logical data type with variable length and 64-bit offsets.
     ///
+    /// Logically the same as [`LargeList`], but the internal representation differs in how child
+    /// data is referenced, allowing flexibility in how data is layed out.
+    ///
     /// Note this data type is not yet fully supported. Using it with arrow APIs may result in `panic`s.
     ///
-    /// The LargeListView layout is defined by three buffers:
-    /// a validity bitmap, an offsets buffer, and an additional sizes buffer.
-    /// Sizes and offsets are both 64 bits for this type
+    /// [`LargeList`]: Self::LargeList
     LargeListView(FieldRef),
     /// A nested datatype that contains a number of sub-fields.
     Struct(Fields),
@@ -552,16 +570,21 @@ impl DataType {
         matches!(self, Int16 | Int32 | Int64)
     }
 
-    /// Returns true if this type is nested (List, FixedSizeList, LargeList, Struct, Union,
+    /// Returns true if this type is nested (List, FixedSizeList, LargeList, ListView. LargeListView, Struct, Union,
     /// or Map), or a dictionary of a nested type
     #[inline]
     pub fn is_nested(&self) -> bool {
         use DataType::*;
         match self {
             Dictionary(_, v) => DataType::is_nested(v.as_ref()),
-            List(_) | FixedSizeList(_, _) | LargeList(_) | Struct(_) | Union(_, _) | Map(_, _) => {
-                true
-            }
+            List(_)
+            | FixedSizeList(_, _)
+            | LargeList(_)
+            | ListView(_)
+            | LargeListView(_)
+            | Struct(_)
+            | Union(_, _)
+            | Map(_, _) => true,
             _ => false,
         }
     }
@@ -578,7 +601,9 @@ impl DataType {
     pub fn equals_datatype(&self, other: &DataType) -> bool {
         match (&self, other) {
             (DataType::List(a), DataType::List(b))
-            | (DataType::LargeList(a), DataType::LargeList(b)) => {
+            | (DataType::LargeList(a), DataType::LargeList(b))
+            | (DataType::ListView(a), DataType::ListView(b))
+            | (DataType::LargeListView(a), DataType::LargeListView(b)) => {
                 a.is_nullable() == b.is_nullable() && a.data_type().equals_datatype(b.data_type())
             }
             (DataType::FixedSizeList(a, a_size), DataType::FixedSizeList(b, b_size)) => {
@@ -725,7 +750,9 @@ impl DataType {
     pub fn contains(&self, other: &DataType) -> bool {
         match (self, other) {
             (DataType::List(f1), DataType::List(f2))
-            | (DataType::LargeList(f1), DataType::LargeList(f2)) => f1.contains(f2),
+            | (DataType::LargeList(f1), DataType::LargeList(f2))
+            | (DataType::ListView(f1), DataType::ListView(f2))
+            | (DataType::LargeListView(f1), DataType::LargeListView(f2)) => f1.contains(f2),
             (DataType::FixedSizeList(f1, s1), DataType::FixedSizeList(f2, s2)) => {
                 s1 == s2 && f1.contains(f2)
             }
@@ -1006,11 +1033,16 @@ mod tests {
     #[test]
     fn test_nested() {
         let list = DataType::List(Arc::new(Field::new("foo", DataType::Utf8, true)));
+        let list_view = DataType::ListView(Arc::new(Field::new("foo", DataType::Utf8, true)));
+        let large_list_view =
+            DataType::LargeListView(Arc::new(Field::new("foo", DataType::Utf8, true)));
 
         assert!(!DataType::is_nested(&DataType::Boolean));
         assert!(!DataType::is_nested(&DataType::Int32));
         assert!(!DataType::is_nested(&DataType::Utf8));
         assert!(DataType::is_nested(&list));
+        assert!(DataType::is_nested(&list_view));
+        assert!(DataType::is_nested(&large_list_view));
 
         assert!(!DataType::is_nested(&DataType::Dictionary(
             Box::new(DataType::Int32),
