@@ -90,8 +90,8 @@ impl<'a> Parser<'a> {
         self.expect_token(Token::LParen)?;
         let data_type = self.parse_next_type()?;
         self.expect_token(Token::RParen)?;
-        Ok(DataType::List(Arc::new(Field::new(
-            "item", data_type, true,
+        Ok(DataType::List(Arc::new(Field::new_list_field(
+            data_type, true,
         ))))
     }
 
@@ -100,8 +100,8 @@ impl<'a> Parser<'a> {
         self.expect_token(Token::LParen)?;
         let data_type = self.parse_next_type()?;
         self.expect_token(Token::RParen)?;
-        Ok(DataType::LargeList(Arc::new(Field::new(
-            "item", data_type, true,
+        Ok(DataType::LargeList(Arc::new(Field::new_list_field(
+            data_type, true,
         ))))
     }
 
@@ -113,7 +113,7 @@ impl<'a> Parser<'a> {
         let data_type = self.parse_next_type()?;
         self.expect_token(Token::RParen)?;
         Ok(DataType::FixedSizeList(
-            Arc::new(Field::new("item", data_type, true)),
+            Arc::new(Field::new_list_field(data_type, true)),
             length,
         ))
     }
